VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CEvalLine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Public Enum Increments
    incOne = 1
    incTwo = 2
    incFour = 4
    incFive = 5
    incTen = 10
    incTwenty = 20
    incOneHundred = 100
End Enum

Public Enum EvalResultType
    rtnChanged = -1
    rtnNoChange
End Enum

Public Enum ActionType
    actRemove = 0
    actAdd
End Enum

Private mactAction As ActionType
Private mstrNewLine As String
Private mintIncrement As Integer
Private mlngCounter As Long
Private mblnContinuation As Boolean 'used to indicate the previous line was a continuation
Private mintInSelectCase As Integer 'use because no line # can appear between the Select Case and the first Case, used as counter in case embedded Select case
Private mblnInEnum As Boolean 'use to track if inside an Enum

Private Declare Function IsCharLower Lib "user32" Alias "IsCharLowerA" (ByVal cChar As Byte) As Long

Private Function CheckLnNumber(ByVal Value As String, LnNumber As Long) As Boolean
'This function will check the line to see if it contains a line number already.
'If the line contains a line number, that number will be returned so that the
'number can be removed form the original line then the new number added if necessary.
'Will return false if no line number, True if has line number and lnNumber will be the
'current value of the line number

    Dim strTemp As String
    Dim intPos As Integer
    Dim blnHasLnNumber As Boolean

    strTemp = Trim(Value)
    
    If Len(strTemp) > 0 Then
        'determine if a space is in the line after trimming
        intPos = InStr(1, strTemp, " ", vbTextCompare)
        'remove any characters that would cause the Isnumeric to fail
        If intPos > 1 Then  'space in line
            strTemp = Mid$(strTemp, 1, intPos - 1)
        End If
        
        'Test IsNumeric will return false on all but true number values
        'i.e. "100Hello", "100:" will fail
        If IsNumeric(strTemp) Then
            If Val(strTemp) >= 0 Then
                blnHasLnNumber = True
                LnNumber = Val(strTemp)
            End If
        End If
    
    End If
            
    CheckLnNumber = blnHasLnNumber

End Function

Private Function CheckRemInLine(ByVal Value As String) As Integer

    Dim intRtn As Integer
    
    intRtn = InStr(1, Value, "'", vbTextCompare)
    If intRtn = 0 Then
        intRtn = InStr(1, Value, "Rem ", vbBinaryCompare)
    End If
    
    CheckRemInLine = intRtn
    
End Function

Private Function IsConditional(Value As String) As Boolean

    Dim blnRtn As Boolean
    
    blnRtn = False
    If StrComp(Left$(Value, 1), "#", vbTextCompare) = 0 Then
        blnRtn = True
    End If
    
    IsConditional = blnRtn
    
End Function

Private Function IsEnum(Value As String) As Boolean

    Dim blnRtn As Boolean
    
    blnRtn = False
    
    If InStr(1, Value, "Enum ", vbBinaryCompare) <> 0 Then    'is Enum
        If InStr(1, Value, "Public Enum ", vbBinaryCompare) = 1 Then    'is Enum
            blnRtn = True
        ElseIf InStr(1, Value, "Public Enum ", vbBinaryCompare) = 1 Then    'is Enum
            blnRtn = True
        ElseIf InStr(1, Value, "Enum ", vbBinaryCompare) = 1 Then   'is Enum
            blnRtn = True
        End If
    End If
    
    If blnRtn = True Then mblnInEnum = True
    
    IsEnum = blnRtn
    
End Function

Private Function IsFirstCase(Value As String) As Boolean
'Call only if the mblnIsSelectCase flag is true!!
'Used to determine if the line is the first case statement after the Selec Case line - can not place line numbers there

    Dim blnRtn As Boolean
    Dim intLen As Integer
    Dim intMinLen As Integer
    
    blnRtn = False
    intLen = Len(Value)
    intMinLen = Len("Case ")
    
    If mintInSelectCase > 0 Then
        If intLen > intMinLen Then
            If StrComp(Left$(Value, intMinLen), "Case ", vbTextCompare) = 0 Then
                blnRtn = True
            End If
        End If
    End If

    IsFirstCase = blnRtn

End Function

Private Sub CheckSelectCase(Value As String)
'Used to set a flag indicating the the select case has been entered but the first case statement
'has not been incountered.  If Case statment can not have a line number

    Dim intLen As Integer
    Dim intMinLen As Integer
    
    intLen = Len(Value)
    intMinLen = Len("Select Case ")
    
    If intLen > intMinLen Then
        If StrComp(Left$(Value, intMinLen), "Select Case ", vbTextCompare) = 0 Then
            mintInSelectCase = mintInSelectCase + 1
        End If
    End If
    
End Sub

Private Sub CheckContinue(Value As String)
'Checks to see if the passed string ends with the line continuation directive (' _') and sets the flag
'variable accordingly

'***** Note that rem lines can be continued with the _
 character so treat all lines that end with _
as if they are continuations

    If Len(Value) > 2 Then
        If (StrComp(Right$(Value, 2), " _", vbTextCompare) = 0) Then
            mblnContinuation = True
        Else
            mblnContinuation = False
        End If
    ElseIf Len(Value) = 1 Then
        If StrComp(Value, "_", vbTextCompare) = 0 Then
            mblnContinuation = True
        Else
            mblnContinuation = False
        End If
    Else
        mblnContinuation = False
    End If
    
End Sub

Private Function IsDeclare(Value As String) As Boolean

    Dim blnRtn As Boolean
    Dim intLen As Integer
    
    On Error GoTo Errhandler
    
    blnRtn = False
    
    If InStr(1, Value, "Dim ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Public ", vbBinaryCompare) = 1 Then
        blnRtn = True
        If InStr(1, Value, "Public Enum ", vbBinaryCompare) = 1 Then
            mblnInEnum = True
        End If
    ElseIf InStr(1, Value, "Private ", vbBinaryCompare) = 1 Then
        blnRtn = True
        If InStr(1, Value, "Private Enum ", vbBinaryCompare) = 1 Then
            mblnInEnum = True
        End If
    ElseIf InStr(1, Value, "Friend ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Static ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Declare ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Option ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Const ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Enum ", vbBinaryCompare) = 1 Then
        blnRtn = True
        mblnInEnum = True
    ElseIf InStr(1, Value, "Implements ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Event ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Type ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Global ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "WithEvents ", vbBinaryCompare) = 1 Then
        blnRtn = True
    ElseIf InStr(1, Value, "Def", vbBinaryCompare) = 1 Then
        If InStr(4, Value, "Bool ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Byte ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Int ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Lng ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Cur ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Sng ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Dbl ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Dec ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Date ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Str ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Obj ", vbBinaryCompare) = 4 Then
            blnRtn = True
        ElseIf InStr(4, Value, "Var ", vbBinaryCompare) = 4 Then
            blnRtn = True
        End If
    Else    'Check for the the definition ie. 'X as Integer' inside type & Enum
        blnRtn = Is_AS_Definition(Value)
    End If
    
    IsDeclare = blnRtn
    
 Exit Function

Errhandler:

    Debug.Assert Err = 0
    Resume Next
End Function

Private Function IsLowerCase(Value As String) As Boolean

    IsLowerCase = CBool(IsCharLower(CByte(Asc(Left$(Value, 1)))))
    
End Function

Private Function IsProcedure(Value As String) As Boolean

    Const MinLen = 7 'Len "Sub I()"
    
    Dim blnRtn As Boolean
    Dim intLen As Integer
    Dim strSearch As String
    
    On Error GoTo Errhandler
    
    blnRtn = False
        
    intLen = Len(Value)
    
    'Now search for Scope directives first
    If intLen >= MinLen Then    'check before searching
            
        If InStr(1, Value, "Sub ", vbTextCompare) = 1 Then
            blnRtn = True
            
        ElseIf InStr(1, Value, "Function ", vbTextCompare) = 1 Then
            blnRtn = True
            
        ElseIf InStr(1, Value, "Property", vbTextCompare) = 1 Then
            blnRtn = True
        
        End If
    End If
    
    IsProcedure = blnRtn
    
 Exit Function

Errhandler:

    Debug.Assert Err = 0
    Resume Next
End Function

Private Function IsEndProcedure(Value As String) As Boolean

    Const MinLen = 5    'Len("End ?")
    
    Dim blnRtn As Boolean
    Dim intLen As Integer
    
    On Error GoTo Errhandler
    
    blnRtn = False
        
    intLen = Len(Value)
    
    'Now search for Scope directives first
    If intLen >= MinLen Then    'check before searching the line must be at least "End ?" long
        If StrComp(Left$(Value, MinLen - 1), "End ", vbBinaryCompare) = 0 Then  'see if this line starts with the "end "
            If InStr(MinLen, Value, "Sub", vbTextCompare) = 5 Then
                blnRtn = True
            ElseIf InStr(MinLen, Value, "Function", vbBinaryCompare) = 5 Then
                blnRtn = True
            ElseIf InStr(MinLen, Value, "Property", vbBinaryCompare) = 5 Then
                blnRtn = True
            ElseIf InStr(MinLen, Value, "Type", vbBinaryCompare) = 5 Then
                blnRtn = True
            ElseIf InStr(MinLen, Value, "Enum", vbBinaryCompare) = 5 Then
                mblnInEnum = False
                blnRtn = True
            End If
        End If
    End If
    
    IsEndProcedure = blnRtn
    
 Exit Function

Errhandler:

    Debug.Assert Err = 0
    Resume Next
    
End Function


Private Function IsRem(Value As String) As Boolean

    Dim blnRtn As Boolean
    
    blnRtn = False
    
    If Len(Value) Then
        If InStr(1, Value, "'", vbTextCompare) = 1 Then
            blnRtn = True
        Else
            If InStr(1, Value, "Rem ", vbTextCompare) = 1 Then
                blnRtn = True
            End If
        End If
    
    End If
    
    IsRem = blnRtn
    
End Function

Public Property Get LnCount() As Long

    If mactAction = actRemove Then
        LnCount = 0
    Else
        LnCount = mlngCounter
    End If
    
End Property

Private Property Let Action(ByVal Value As ActionType)

    Select Case Value
        Case actAdd, actRemove
        Case Else
            Err.Raise 13
    End Select
    
    mactAction = Value
    
End Property

Public Property Get Action() As ActionType

    Action = mactAction
    
End Property

Public Property Get NewLine() As String
    
    NewLine = mstrNewLine
    
End Property

Private Property Let NewLine(ByVal strNewLine As String)

    mstrNewLine = strNewLine
    
End Property

Public Function EvaluateLine(ByVal strCodeLine As String, _
        ByVal DoAction As ActionType _
        ) As EvalResultType

    Dim intPos As Integer
    
    'set the action and increment values
    Action = DoAction
    
    'Always place the original string into the NewLine even though nochange may occur
    NewLine = strCodeLine
        
    If Action = actAdd Then
        EvaluateLine = EvalAddLn(strCodeLine)
    Else 'actRemove
        EvaluateLine = EvalRemoveLn(strCodeLine)
    End If
    
End Function

Public Property Let Increment(ByVal Value As Increments)

    Select Case Value
        Case incOne, incTwo, incFour, incFive, incTen, incTwenty, incOneHundred
        Case Else
            Err.Raise 13
    End Select
    
    mintIncrement = Value
    mlngCounter = mintIncrement
    
End Property

Public Property Get Increment() As Increments

    Increment = mintIncrement
    
End Property


Private Sub Class_Initialize()

    mintIncrement = incTen
    mlngCounter = mintIncrement
    
End Sub

Private Sub Inc()

    mlngCounter = mlngCounter + mintIncrement
    
End Sub

Private Function EvalAddLn(ByVal strCodeLn As String) As EvalResultType
'This function will evaluate the code line and add or change a line number if
'the line can/should have a line number. Will also return a value indicating
'whether the line has changed and set the Newline property if it has
    
    Dim strTemp As String
    Dim enmRtn As EvalResultType
    Dim lngLnNum As Long
    Dim strFormat As String
    Dim intKeyCode As Integer

    On Error GoTo Errhandler
        
'    NewLine = strCodeLn 'save the passed line in NewLine
    
    enmRtn = rtnNoChange
    EvalAddLn = enmRtn
    
    strTemp = Trim$(strCodeLn) 'copy string
    
    If Len(strTemp) > 0 Then    'must have content
    
        If Not mblnContinuation Then    'if the continuation flag is set skip this line because continue code can ot have a line number
        
            '--------------------------------------------------------------------------------------------------------
            'The check for a line number is first because a line number may appear on an empty, code, or comment line.
            'check if line already has a line number
            If CheckLnNumber(strTemp, lngLnNum) Then  'if the returned value is not zero then the line had a line number
                'the following will remove the line number from the original string stored in the newline
                'property.
                If Len(strCodeLn) > Len(CStr(lngLnNum)) Then    'check the the line does not contain only the line number
                    NewLine = Mid$(strCodeLn, Len(CStr(lngLnNum)) + 1)
                Else    'contained only the line number so it should be empty
                    NewLine = ""
                End If
                enmRtn = rtnChanged 'set the changed return flag
                strTemp = Trim$(NewLine)    'set the check string to the newline trimmed
                If Len(strTemp) = 0 Then
                    EvalAddLn = enmRtn
                    Exit Function   'if removing the line number made an empty string, exit!
                End If
            End If
            '--------------------------------------------------------------------------------------------------------
            If Not IsRem(strTemp) Then  'check for remark
                CheckContinue strTemp   'Ok the last line was not a continuation but is this line so that continue flag can be set
                If Not IsLabelLine(strTemp) Then
                    If Not IsLowerCase(strTemp) Then
                        If Not IsDeclare(strTemp) Then  'check the line is not a declaration line
                            If Not IsProcedure(strTemp) Then
                                If Not IsEndProcedure(strTemp) Then
                                    If Not IsConditional(strTemp) Then  'check the line does not have the #
                                        If (mintInSelectCase > 0) And IsFirstCase(strTemp) Then
                                            mintInSelectCase = mintInSelectCase - 1  'decrement the in select case counter so next line not checked if
                                        Else    'This line needs a line number!!!
                                                                                        
                                            CheckSelectCase strTemp   'This will increment the counter if this is a select case line
                                            If Not mblnInEnum Then
                                                'Check to see if a space should follow the line number in case
                                                'the code is right has no space or tab before
                                                intKeyCode = Asc(Left(NewLine, 1))
                                                If (intKeyCode = vbKeyTab) Or (intKeyCode = vbKeySpace) Then
                                                    strFormat = "0"
                                                Else
                                                    strFormat = "0 "
                                                End If
                                                
                                                NewLine = Format$(mlngCounter, strFormat) & NewLine
                                                enmRtn = rtnChanged 'set changed flag
                                                Inc 'increment the line number
                                            End If
                                        End If
                                    End If
                                End If
                            End If
                        End If
                    Else    'starts with a lower case letter
                        If Not Is_AS_Definition(strTemp) Then
                            If Not IsEnum(strTemp) Then
                                If Not mblnInEnum Then
                                    'Check to see if a space should follow the line number in case
                                    'the code is right has no space or tab before
                                    intKeyCode = Asc(Left(NewLine, 1))
                                    If (intKeyCode = vbKeyTab) Or (intKeyCode = vbKeySpace) Then
                                        strFormat = "0"
                                    Else
                                        strFormat = "0 "
                                    End If
                                                
                                    NewLine = Format$(mlngCounter, strFormat) & NewLine
                                    
                                    enmRtn = rtnChanged 'set changed flag
                                    Inc 'increment the line number
                                End If
                            End If
                        End If
                    End If
                End If
            End If
            
        Else    'skipped because previous line had continuation character but this line should be check to see if it is a continuation line as well
                'or is it the last line of the continuation
            CheckContinue strTemp
            
        End If
    
    End If
    
    EvalAddLn = enmRtn
    

 Exit Function

Errhandler:
    Debug.Assert Err = 0
    Resume Next
    
End Function

Private Function EvalRemoveLn(ByVal strCodeLn As String) As EvalResultType
'This function will evaluate the code line and remove a line number if
'the line has a line number. Will also return a value indicating
'whether the line has changed and set the Newline property if it has

    Dim strTemp As String
    Dim enmRtn As EvalResultType
    Dim lngLnNum As Long
    
    On Error GoTo Errhandler
    
    enmRtn = rtnNoChange
    EvalRemoveLn = enmRtn

    strTemp = Trim$(strCodeLn) 'copy string

    If Len(strTemp) > 0 Then    'must have content

        If Not mblnContinuation Then    'if the continuation flag is set skip this line because continue code can ot have a line number
            '--------------------------------------------------------------------------------------------------------
            'The check for a line number
            'check if line already has a line number
            If CheckLnNumber(strTemp, lngLnNum) Then  'if the returned value is not zero then the line had a line number
                'the following will remove the line number from the original string stored in the newline
                'property.
                If Len(strCodeLn) > Len(CStr(lngLnNum)) Then    'check the the line does not contain only the line number
                    NewLine = Mid$(strCodeLn, Len(CStr(lngLnNum)) + 1)
                Else    'contained only the line number so it should be empty
                    NewLine = ""
                End If
                enmRtn = rtnChanged 'set the changed return flag

            End If
            '--------------------------------------------------------------------------------------------------------
        Else    'skipped because previous line had continuation character but this line should be check to see if it is a continuation line as well
                'or is it the last line of the continuation
            CheckContinue strTemp
        End If
        

    End If
    
    EvalRemoveLn = enmRtn

 Exit Function

Errhandler:
    Debug.Assert Err = 0
    Resume Next

End Function

Private Function Is_AS_Definition(Value As String) As Boolean
    
    Dim blnRtn As Boolean
    Dim intRemPos As Integer
    Dim intPosAs As Integer

    blnRtn = False
    intRemPos = CheckRemInLine(Value)

    intPosAs = InStr(1, Value, " As ", vbBinaryCompare)   'If this is actually a definition the word "As" has a capitol "A"
    If intPosAs > 1 Then
        If (intRemPos > 0) Then
            If intPosAs < intRemPos Then
                blnRtn = True
            End If
        Else
            blnRtn = True
        End If
    End If
        
    Is_AS_Definition = blnRtn
    

End Function

Private Function IsLabelLine(ByVal Value As String) As Boolean

    Dim blnRtn As Boolean
    Dim strTemp As String
    Dim intPos As Integer
    
    blnRtn = False  'default false
    strTemp = Trim(Value)
    
    If Len(strTemp) > 0 Then
        intPos = InStr(1, strTemp, " ", vbTextCompare)
        If intPos >= 2 Then
            strTemp = Left$(strTemp, intPos - 1)
        End If
        If StrComp(Right(strTemp, 1), ":", vbTextCompare) = 0 Then
            blnRtn = True
        End If
    End If
    
    IsLabelLine = blnRtn
    
End Function
